using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.Reporting;
using CMS.GlobalHelper;
using CMS.EventLog;
using CMS.DataEngine;
using CMS.CMSHelper;

public partial class CMSReporting_ReportTable : AbstractReportControl
{
    private string mReportTableName = "";

    private GridView gvTable = null;

        /// <summary>
    /// Table name
    /// </summary>
    private string ReportTableName
    {
        get
        {
            return mReportTableName;
        }
        set
        {
            mReportTableName = value;
        }
    }

    
    /// <summary>
    /// Reload data
    /// </summary>
    public override void ReloadData(bool forceLoad)
    {
        try
        {
            // Set table name from inline parameter
            this.ReportTableName = this.Parameter;

            // Get report table info
            ReportTableInfo rti = ReportTableInfoProvider.GetReportTableInfo(this.ReportTableName);
            if (rti != null)
            {
                this.QueryIsStoredProcedure = rti.TableQueryIsStoredProcedure;
                this.QueryText = ResolveMacros(rti.TableQuery);

                // Setup grid view in codebehind, so SkinID can be set
                gvTable = new GridView();
                gvTable.AllowPaging = false;
                gvTable.AllowPaging = false;
                gvTable.AllowSorting = false;
                gvTable.CellPadding = 3;
                gvTable.GridLines = GridLines.Horizontal;
                gvTable.HeaderStyle.HorizontalAlign = HorizontalAlign.Left;

                // Get SkinID from reportTable custom data
                string skinId = ValidationHelper.GetString(rti.TableSettings["skinid"], "");
                if (skinId != String.Empty)
                {
                    gvTable.SkinID = skinId;   
                }

                // Add grid view control to the page
                this.plcGrid.Controls.Clear();
                this.plcGrid.Controls.Add(gvTable);
            }

            // Only use base parameters in case of stored procedure
            if (this.QueryIsStoredProcedure)
            {
                this.AllParameters = SpecialFunctions.ConvertDataRowToParams(this.ReportParameters, null);
            }

            // Load data
            DataSet ds = this.LoadData();

            // If no data load, set empty dataset
            if (DataHelper.DataSourceIsEmpty(ds))
            {
                ds = new DataSet();
                ds.Tables.Add();
            }
            else
            {
                // Resolve macros in column names
                int i = 0;
                foreach (DataColumn dc in ds.Tables[0].Columns)
                {
                    if (dc.ColumnName == "Column" + ((int)(i + 1)).ToString())
                    {
                        dc.ColumnName = ResolveMacros(ds.Tables[0].Rows[0][i].ToString());
                    }
                    else
                    {
                        dc.ColumnName = ResolveMacros(dc.ColumnName);
                    }
                    i++;
                }

                // Resolve macros in dataset
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    foreach (DataColumn dc in ds.Tables[0].Columns)
                    {
                        if (dc.DataType.FullName.ToLower() == "system.string")
                        {
                            dr[dc.ColumnName] = ResolveMacros(ValidationHelper.GetString(dr[dc.ColumnName], ""));
                        }
                    }
                }
            }

            // Databind to gridview control
            gvTable.DataSource = ds;
            gvTable.DataBind();

            
        }
        catch (Exception ex)
        {
            // Display error message, if data load fail
            lblError.Visible = true;
            lblError.Text = "[ReportTable.ascx] Error loading the data: " + ex.Message;
            EventLogProvider ev = new EventLogProvider();
            ev.LogEvent("Report table", "E", ex);
        }
    }
}
